@@ -1,22 +1,22 @@
 /*
 
-Copyright (C) 2011-2016 Michael Goffioul
+Copyright (C) 2011-2019 Michael Goffioul
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
@@ -41,6 +41,10 @@ along with Octave; see the file COPYING.  If not, see
 #include <QtDebug>
 #include <QTimer>
 #include <QToolBar>
+#if defined (HAVE_QSCREEN_DEVICEPIXELRATIO)
+#  include <QWindow>
+#  include <QScreen>
+#endif
 
 #include "Canvas.h"
 #include "Container.h"
@@ -63,43 +67,6 @@ namespace QtHandles
 
   DECLARE_GENERICEVENTNOTIFY_SENDER(MenuBar, QMenuBar);
 
-  static bool
-  hasUiControlChildren (const figure::properties& fp)
-  {
-    gh_manager::auto_lock lock;
-
-    Matrix kids = fp.get_all_children ();
-
-    for (int i = 0; i < kids.numel (); i++)
-      {
-        graphics_object go (gh_manager::get_object (kids(i)));
-
-        if (go && (go.isa ("uicontrol") || go.isa ("uipanel")
-                   || go.isa ("uibuttongroup")))
-          return true;
-      }
-
-    return false;
-  }
-
-  static bool
-  hasUiMenuChildren (const figure::properties& fp)
-  {
-    gh_manager::auto_lock lock;
-
-    Matrix kids = fp.get_all_children ();
-
-    for (int i = 0; i < kids.numel (); i++)
-      {
-        graphics_object go (gh_manager::get_object (kids(i)));
-
-        if (go && go.isa ("uimenu"))
-          return true;
-      }
-
-    return false;
-  }
-
   static QRect
   boundingBoxToRect (const Matrix& bb)
   {
@@ -122,15 +89,21 @@ namespace QtHandles
     return new Figure (go, new FigureWindow ());
   }
 
-  Figure::Figure (const graphics_object& go, FigureWindow* win)
-    : Object (go, win), m_blockUpdates (false), m_figureToolBar (0),
-      m_menuBar (0), m_innerRect (), m_outerRect (), m_mouseModeGroup (0)
+  Figure::Figure (const graphics_object& go, FigureWindow *win)
+    : Object (go, win), m_blockUpdates (false), m_figureToolBar (nullptr),
+      m_menuBar (nullptr), m_innerRect (), m_outerRect (),
+      m_mouseModeGroup (nullptr), m_previousHeight (0), m_resizable (true)
   {
     m_container = new Container (win);
     win->setCentralWidget (m_container);
 
     figure::properties& fp = properties<figure> ();
 
+    // Register for the signal that indicates when a window has moved
+    // to a different screen
+    connect (win, SIGNAL (figureWindowShown ()),
+             this, SLOT (figureWindowShown ()));
+
     // Status bar
     m_statusBar = win->statusBar ();
     int boffset = 0;
@@ -139,9 +112,8 @@ namespace QtHandles
     createFigureToolBarAndMenuBar ();
     int toffset = 0;
 
-    if (fp.toolbar_is ("figure") ||
-        (fp.toolbar_is ("auto") && fp.menubar_is ("figure") &&
-         ! hasUiControlChildren (fp)))
+    if (fp.toolbar_is ("figure")
+        || (fp.toolbar_is ("auto") && fp.menubar_is ("figure")))
       {
         toffset += m_figureToolBar->sizeHint ().height ();
         boffset += m_statusBar->sizeHint ().height ();
@@ -152,15 +124,10 @@ namespace QtHandles
         m_statusBar->hide ();
       }
 
-    if (fp.menubar_is ("figure") || hasUiMenuChildren (fp))
-      toffset += m_menuBar->sizeHint ().height ();
-    else
-      m_menuBar->hide ();
-
     m_innerRect = boundingBoxToRect (fp.get_boundingbox (true));
     m_outerRect = boundingBoxToRect (fp.get_boundingbox (false));
 
-    win->setGeometry (m_innerRect.adjusted (0, -toffset, 0, boffset));
+    set_geometry (m_innerRect.adjusted (0, -toffset, 0, boffset));
 
     // Enable mouse tracking unconditionally
     enableMouseTracking ();
@@ -179,6 +146,9 @@ namespace QtHandles
     // modal style
     update (figure::properties::ID_WINDOWSTYLE);
 
+    // Handle resizing constraints
+    update (figure::properties::ID_RESIZE);
+
     // Visibility
     update (figure::properties::ID_VISIBLE);
 
@@ -215,9 +185,6 @@ namespace QtHandles
       case TextMode:
         return "text";
 
-      case SelectMode:
-        return "select";
-
       default:
         break;
       }
@@ -240,8 +207,6 @@ namespace QtHandles
       return PanMode;
     else if (mode == "text")
       return TextMode;
-    else if (mode == "select")
-      return SelectMode;
     else
       return NoMode;
   }
@@ -283,7 +248,7 @@ namespace QtHandles
 
         std::string direction = zm.getfield ("Direction").string_value ();
 
-        mode += " " + direction;
+        mode += ' ' + direction;
       }
 
     return mouse_mode_from_string (mode);
@@ -292,7 +257,7 @@ namespace QtHandles
   void
   Figure::createFigureToolBarAndMenuBar (void)
   {
-    QMainWindow* win = qWidget<QMainWindow> ();
+    QMainWindow *win = qWidget<QMainWindow> ();
 
     m_figureToolBar = win->addToolBar (tr ("Figure ToolBar"));
     m_figureToolBar->setMovable (false);
@@ -317,27 +282,6 @@ namespace QtHandles
 
     m_menuBar = new MenuBar (win);
     win->setMenuBar (m_menuBar);
-
-    QMenu* fileMenu = m_menuBar->addMenu (tr ("&File"));
-    fileMenu->menuAction ()->setObjectName ("builtinMenu");
-    fileMenu->addAction (tr ("&Save"), this, SLOT (fileSaveFigure (bool)));
-    fileMenu->addAction (tr ("Save &As"), this, SLOT (fileSaveFigureAs (void)));
-    fileMenu->addSeparator ();
-    fileMenu->addAction (tr ("&Close Figure"), this,
-                         SLOT (fileCloseFigure (void)), Qt::CTRL | Qt::Key_W);
-
-    QMenu* editMenu = m_menuBar->addMenu (tr ("&Edit"));
-    editMenu->menuAction ()->setObjectName ("builtinMenu");
-    editMenu->addAction (tr ("Cop&y"), this, SLOT (editCopy (bool)),
-                         Qt::CTRL | Qt::Key_C);
-    editMenu->addSeparator ();
-    editMenu->addActions (m_mouseModeGroup->actions ());
-
-    QMenu* helpMenu = m_menuBar->addMenu (tr ("&Help"));
-    helpMenu->menuAction ()->setObjectName ("builtinMenu");
-    helpMenu->addAction (tr ("About Octave"), this,
-                         SLOT (helpAboutOctave (void)));
-
     m_menuBar->addReceiver (this);
   }
 
@@ -352,6 +296,26 @@ namespace QtHandles
       }
   }
 
+  void
+  Figure::set_geometry (QRect r)
+  {
+    QMainWindow *win = qWidget<QMainWindow> ();
+
+    if (! m_resizable)
+      {
+        win->setSizePolicy (QSizePolicy::Preferred, QSizePolicy::Preferred);
+        win->setFixedSize (QSize( QWIDGETSIZE_MAX, QWIDGETSIZE_MAX));
+      }
+
+    win->setGeometry (r);
+
+    if (! m_resizable)
+      {
+        win->setSizePolicy (QSizePolicy::Fixed, QSizePolicy::Fixed);
+        win->setFixedSize(win->size ());
+      }
+  }
+
   Container*
   Figure::innerContainer (void)
   {
@@ -361,43 +325,65 @@ namespace QtHandles
   void
   Figure::redraw (void)
   {
-    Canvas* canvas = m_container->canvas (m_handle);
+    Canvas *canvas = m_container->canvas (m_handle);
 
     if (canvas)
-      {
-        canvas->redraw ();
-        //canvas->setMouseMode (RotateMode);
-      }
+      canvas->redraw ();
 
-    foreach (QFrame* frame,
+    foreach (QFrame *frame,
              qWidget<QWidget> ()->findChildren<QFrame*> ())
-    {
-      if (frame->objectName () == "UIPanel"
-          || frame->objectName () == "UIButtonGroup")
-        {
-          Object* obj = Object::fromQObject (frame);
+      {
+        if (frame->objectName () == "UIPanel"
+            || frame->objectName () == "UIButtonGroup")
+          {
+            Object *obj = Object::fromQObject (frame);
 
-          if (obj)
-            obj->slotRedraw ();
-        }
-    }
+            if (obj)
+              obj->slotRedraw ();
+          }
+      }
 
     updateFigureToolBarAndMenuBar ();
   }
 
+  void
+  Figure::show (void)
+  {
+    QWidget *win = qWidget<QWidget> ();
+
+    win->activateWindow ();
+    win->raise ();
+  }
+
   void
   Figure::print (const QString& file_cmd, const QString& term)
   {
-    Canvas* canvas = m_container->canvas (m_handle);
+    Canvas *canvas = m_container->canvas (m_handle);
 
     if (canvas)
       canvas->print (file_cmd, term);
   }
 
+  uint8NDArray
+  Figure::slotGetPixels (void)
+  {
+    uint8NDArray retval;
+    Canvas *canvas = m_container->canvas (m_handle);
+
+    if (canvas)
+      {
+        gh_manager::process_events ();
+        gh_manager::auto_lock lock;
+        retval = canvas->getPixels ();
+      }
+
+    return retval;
+  }
+
   void
   Figure::beingDeleted (void)
   {
-    Canvas* canvas = m_container->canvas (m_handle.value (), false);
+    Canvas *canvas = m_container->canvas (m_handle.value (), false);
 
     if (canvas)
       canvas->blockRedraw (true);
@@ -414,7 +400,15 @@ namespace QtHandles
       return;
 
     figure::properties& fp = properties<figure> ();
-    QMainWindow* win = qWidget<QMainWindow> ();
+
+    if (fp.is___printing__ ())
+      return;
+
+    QMainWindow *win = qWidget<QMainWindow> ();
+
+    // If the window doesn't exist, there's nothing we can do.
+    if (! win)
+      return;
 
     m_blockUpdates = true;
 
@@ -426,9 +420,9 @@ namespace QtHandles
           int toffset = 0;
           int boffset = 0;
 
-          foreach (QToolBar* tb, win->findChildren<QToolBar*> ())
-          if (! tb->isHidden ())
-            toffset += tb->sizeHint ().height ();
+          foreach (QToolBar *tb, win->findChildren<QToolBar*> ())
+            if (! tb->isHidden ())
+              toffset += tb->sizeHint ().height ();
 
           if (! m_menuBar->isHidden ())
             toffset += m_menuBar->sizeHint ().height ();
@@ -436,7 +430,7 @@ namespace QtHandles
           if (! m_statusBar->isHidden ())
             boffset += m_statusBar->sizeHint ().height ();
 
-          win->setGeometry (m_innerRect.adjusted (0, -toffset, 0, boffset));
+          set_geometry (m_innerRect.adjusted (0, -toffset, 0, boffset));
         }
         break;
 
@@ -447,36 +441,56 @@ namespace QtHandles
 
       case figure::properties::ID_VISIBLE:
         if (fp.is_visible ())
-          QTimer::singleShot (0, win, SLOT (show ()));
+          {
+            QTimer::singleShot (0, win, SLOT (show ()));
+            if (! fp.is___gl_window__ ())
+              {
+                gh_manager::auto_lock lock;
+                fp.set ("__gl_window__", "on");
+              }
+          }
         else
           win->hide ();
         break;
 
+      case figure::properties::ID_RESIZE:
+        if (fp.is_resize ())
+          {
+            win->setSizePolicy (QSizePolicy::Preferred, QSizePolicy::Preferred);
+            win->setFixedSize (QSize( QWIDGETSIZE_MAX, QWIDGETSIZE_MAX));
+            m_resizable = true;
+          }
+        else
+          {
+            win->setSizePolicy (QSizePolicy::Fixed, QSizePolicy::Fixed);
+            win->setFixedSize(win->size ());
+            m_resizable = false;
+          }
+        break;
+
       case figure::properties::ID_TOOLBAR:
         if (fp.toolbar_is ("none"))
           showFigureToolBar (false);
         else if (fp.toolbar_is ("figure"))
           showFigureToolBar (true);
-        else // "auto"
-          showFigureToolBar (! hasUiControlChildren (fp) &&
-                             fp.menubar_is ("figure"));
+        else  // "auto"
+          showFigureToolBar (fp.menubar_is ("figure"));
         break;
 
       case figure::properties::ID_MENUBAR:
-        showMenuBar (fp.menubar_is ("figure"));
         if (fp.toolbar_is ("auto"))
           showFigureToolBar (fp.menubar_is ("figure"));
         break;
 
       case figure::properties::ID_KEYPRESSFCN:
-        if (fp.get_keypressfcn ().is_empty ())
+        if (fp.get_keypressfcn ().isempty ())
           m_container->canvas (m_handle)->clearEventMask (Canvas::KeyPress);
         else
           m_container->canvas (m_handle)->addEventMask (Canvas::KeyPress);
         break;
 
       case figure::properties::ID_KEYRELEASEFCN:
-        if (fp.get_keyreleasefcn ().is_empty ())
+        if (fp.get_keyreleasefcn ().isempty ())
           m_container->canvas (m_handle)->clearEventMask (Canvas::KeyRelease);
         else
           m_container->canvas (m_handle)->addEventMask (Canvas::KeyRelease);
@@ -500,6 +514,10 @@ namespace QtHandles
 
         break;
 
+      case figure::properties::ID___MOUSE_MODE__:
+        m_container->canvas (m_handle)->setCursor (mouseMode ());
+        break;
+
       default:
         break;
       }
@@ -522,7 +540,7 @@ namespace QtHandles
           r.adjust (0, -dy1, 0, dy2);
 
         m_blockUpdates = true;
-        qWidget<QWidget> ()->setGeometry (r);
+        set_geometry (r);
         m_figureToolBar->setVisible (visible);
         m_statusBar->setVisible (visible);
         m_blockUpdates = false;
@@ -532,48 +550,23 @@ namespace QtHandles
   }
 
   void
-  Figure::showMenuBar (bool visible)
-  {
-    int h1 = m_menuBar->sizeHint ().height ();
-
-    foreach (QAction* a, m_menuBar->actions ())
-    if (a->objectName () == "builtinMenu")
-      a->setVisible (visible);
-
-    int h2 = m_menuBar->sizeHint ().height ();
-
-    if (! visible)
-      visible = hasUiMenuChildren (properties<figure> ());
-
-    if (! m_menuBar->isHidden ())
-      {
-        int dy = qMax (h1, h2);
-        QRect r = qWidget<QWidget> ()->geometry ();
-
-        //qDebug () << "Figure::showMenuBar:" << r;
-        if (! visible)
-          r.adjust (0, dy, 0, 0);
-        else
-          r.adjust (0, -dy, 0, 0);
-        //qDebug () << "Figure::showMenuBar(adjusted):" << r;
-
-        m_blockUpdates = true;
-        qWidget<QWidget> ()->setGeometry (r);
-        m_menuBar->setVisible (visible);
-        m_blockUpdates = false;
-
-        updateBoundingBox (false);
-      }
-  }
-
-  void
-  Figure::updateMenuBar (void)
+  Figure::updateFigureHeight (int dh)
   {
     gh_manager::auto_lock lock;
     graphics_object go = object ();
 
-    if (go.valid_object ())
-      showMenuBar (Utils::properties<figure> (go).menubar_is ("figure"));
+    if (go.valid_object () && dh != 0)
+      {
+        QRect r = qWidget<QWidget> ()->geometry ();
+
+        r.adjust (0, dh, 0, 0);
+
+        m_blockUpdates = true;
+        set_geometry (r);
+        m_blockUpdates = false;
+
+        updateBoundingBox (false);
+      }
   }
 
   void
@@ -596,35 +589,19 @@ namespace QtHandles
     Matrix m_bbox;
     bool m_internal;
     graphics_handle m_handle;
-    Figure* m_figure;
+    Figure *m_figure;
   };
 
-  void
-  Figure::updateBoundingBoxHelper (void* data)
-  {
-    gh_manager::auto_lock lock;
-
-    UpdateBoundingBoxData* d = reinterpret_cast<UpdateBoundingBoxData*> (data);
-    graphics_object go = gh_manager::get_object (d->m_handle);
-
-    if (go.valid_object ())
-      {
-        figure::properties& fp = Utils::properties<figure> (go);
-
-        fp.set_boundingbox (d->m_bbox, d->m_internal, false);
-      }
-
-    delete d;
-  }
-
   void
   Figure::updateBoundingBox (bool internal, int flags)
   {
-    QWidget* win = qWidget<QWidget> ();
+    QWidget *win = qWidget<QWidget> ();
     Matrix bb (1, 4);
+    std::string prop;
 
     if (internal)
       {
+        prop = "position";
         QRect r = m_innerRect;
 
         if (flags & UpdateBoundingBoxPosition)
@@ -646,6 +623,7 @@ namespace QtHandles
       }
     else
       {
+        prop = "outerposition";
         QRect r = m_outerRect;
 
         if (flags & UpdateBoundingBoxPosition)
@@ -666,21 +644,27 @@ namespace QtHandles
           return;
       }
 
-    UpdateBoundingBoxData* d = new UpdateBoundingBoxData ();
+    figure::properties& fp = properties<figure> ();
 
-    d->m_bbox = bb;
-    d->m_internal = internal;
-    d->m_handle = m_handle;
-    d->m_figure = this;
-
-    gh_manager::post_function (Figure::updateBoundingBoxHelper, d);
+    gh_manager::post_set (m_handle, prop, fp.bbox2position (bb), false,
+                          prop == "position");
   }
 
   bool
-  Figure::eventNotifyBefore (QObject* obj, QEvent* xevent)
+  Figure::eventNotifyBefore (QObject *obj, QEvent *xevent)
   {
     if (! m_blockUpdates)
       {
+        // Clicking the toolbar or the menubar makes this figure current
+        if (xevent->type () == QEvent::MouseButtonPress)
+          {
+            figure::properties& fp = properties<figure> ();
+            graphics_object root = gh_manager::get_object (0);
+            if (fp.get_handlevisibility () == "on")
+              root.set ("currentfigure",
+                        fp.get___myhandle__ ().as_octave_value ());
+          }
+
         if (obj == m_container)
           {
             // Do nothing...
@@ -689,15 +673,10 @@ namespace QtHandles
           {
             switch (xevent->type ())
               {
+              case QEvent::ActionAdded:
+              case QEvent::ActionChanged:
               case QEvent::ActionRemoved:
-                {
-                  QAction* a = dynamic_cast<QActionEvent*> (xevent)->action ();
-
-                  if (! a->isSeparator ()
-                      && a->objectName () != "builtinMenu")
-                    updateMenuBar ();
-                }
-                break;
+                m_previousHeight = m_menuBar->sizeHint ().height ();
 
               default:
                 break;
@@ -722,7 +701,7 @@ namespace QtHandles
   }
 
   void
-  Figure::eventNotifyAfter (QObject* watched, QEvent* xevent)
+  Figure::eventNotifyAfter (QObject *watched, QEvent *xevent)
   {
     if (! m_blockUpdates)
       {
@@ -735,7 +714,7 @@ namespace QtHandles
                 break;
 
               case QEvent::ChildAdded:
-                if (dynamic_cast<QChildEvent*> (xevent)->child
+                if (dynamic_cast<QChildEvent *> (xevent)->child
                     ()->isWidgetType())
                   {
                     gh_manager::auto_lock lock;
@@ -743,14 +722,16 @@ namespace QtHandles
 
                     enableMouseTracking ();
                   }
+                break;
 
               case QEvent::ChildRemoved:
-                if (dynamic_cast<QChildEvent*> (xevent)->child
+                if (dynamic_cast<QChildEvent *> (xevent)->child
                     ()->isWidgetType())
                   {
                     gh_manager::auto_lock lock;
                     update (figure::properties::ID_TOOLBAR);
                   }
+                break;
 
               default:
                 break;
@@ -761,12 +742,15 @@ namespace QtHandles
             switch (xevent->type ())
               {
               case QEvent::ActionAdded:
+              case QEvent::ActionChanged:
+              case QEvent::ActionRemoved:
+                // The menubar may have been resized if no action is visible
                 {
-                  QAction* a = dynamic_cast<QActionEvent*> (xevent)->action ();
-
-                  if (! a->isSeparator ()
-                      && a->objectName () != "builtinMenu")
-                    updateMenuBar ();
+                  QAction *a = dynamic_cast<QActionEvent *> (xevent)->action ();
+                  int currentHeight = m_menuBar->sizeHint ().height ();
+                  if (currentHeight != m_previousHeight
+                      && ! a->isSeparator ())
+                    updateFigureHeight (m_previousHeight - currentHeight);
                 }
                 break;
 
@@ -794,16 +778,6 @@ namespace QtHandles
       }
   }
 
-  void
-  Figure::helpAboutOctave (void)
-  {
-    std::string message
-      = octave_name_version_copyright_copying_warranty_and_bugs (true);
-
-    QMessageBox::about (qWidget<QMainWindow> (), tr ("About Octave"),
-                        QString::fromStdString (message));
-  }
-
   void
   Figure::setMouseMode (MouseMode mode)
   {
@@ -816,112 +790,16 @@ namespace QtHandles
 
     fp.set___mouse_mode__ (mouse_mode_to_string (mode));
 
-    Canvas* canvas = m_container->canvas (m_handle);
+    Canvas *canvas = m_container->canvas (m_handle);
 
     if (canvas)
       canvas->setCursor (mode);
   }
 
   void
-  Figure::fileSaveFigure (bool prompt)
+  Figure::addCustomToolBar (QToolBar *bar, bool visible)
   {
-    QString file = fileName ();
-
-    if (file.isEmpty ())
-      {
-        prompt = true;
-
-        file = "untitled.pdf";
-      }
-
-    if (prompt || file.isEmpty ())
-      {
-        QFileInfo finfo (file);
-
-        file = QFileDialog::getSaveFileName (qWidget<FigureWindow> (),
-                                             tr ("Save Figure As"),
-                                             finfo.absoluteFilePath (), 0, 0,
-                                             QFileDialog::DontUseNativeDialog);
-      }
-
-    if (! file.isEmpty ())
-      {
-        QFileInfo finfo (file);
-
-        setFileName (finfo.absoluteFilePath ());
-
-        octave_link::post_event (this, &Figure::save_figure_callback,
-                                 file.toStdString ());
-      }
-  }
-
-  void
-  Figure::save_figure_callback (const std::string& file)
-  {
-    figure::properties& fp = properties<figure> ();
-    octave_value fnum = fp.get___myhandle__ ().as_octave_value ();
-
-    Ffeval (ovl ("print", fnum, file));
-  }
-
-  void
-  Figure::copy_figure_callback (const std::string& format)
-  {
-    std::string msg;
-
-    std::string file = octave::sys::tempnam ("", "oct-", msg) + "." + format;
-
-    if (file.empty ())
-      {
-        // Report error?
-        return;
-      }
-
-    save_figure_callback (file);
-
-    octave_link::copy_image_to_clipboard (file);
-  }
-
-  void
-  Figure::fileSaveFigureAs (void)
-  {
-    fileSaveFigure (true);
-  }
-
-  void
-  Figure::fileCloseFigure (void)
-  {
-    qWidget<QMainWindow> ()->close ();
-  }
-
-  void
-  Figure::editCopy (bool /* choose_format */)
-  {
-    QString format = "png";
-
-#if 0
-
-    // FIXME: allow choice of image formats.
-
-    if (choose_format)
-      {
-        QFileInfo finfo (file);
-
-        format = QFileDialog::getSaveFileName (qWidget<FigureWindow> (),
-                                               tr ("Save Figure As"),
-                                               finfo.absoluteFilePath (), 0, 0,
-                                               QFileDialog::DontUseNativeDialog);
-      }
-#endif
-
-    octave_link::post_event (this, &Figure::copy_figure_callback,
-                             format.toStdString ());
-  }
-
-  void
-  Figure::addCustomToolBar (QToolBar* bar, bool visible)
-  {
-    QMainWindow* win = qWidget<QMainWindow> ();
+    QMainWindow *win = qWidget<QMainWindow> ();
 
     if (! visible)
       win->addToolBar (bar);
@@ -934,7 +812,7 @@ namespace QtHandles
         r.adjust (0, -sz.height (), 0, 0);
 
         m_blockUpdates = true;
-        win->setGeometry (r);
+        set_geometry (r);
         win->addToolBarBreak ();
         win->addToolBar (bar);
         m_blockUpdates = false;
@@ -945,9 +823,9 @@ namespace QtHandles
   }
 
   void
-  Figure::showCustomToolBar (QToolBar* bar, bool visible)
+  Figure::showCustomToolBar (QToolBar *bar, bool visible)
   {
-    QMainWindow* win = qWidget<QMainWindow> ();
+    QMainWindow *win = qWidget<QMainWindow> ();
 
     if ((! bar->isHidden ()) != visible)
       {
@@ -960,7 +838,7 @@ namespace QtHandles
           r.adjust (0, sz.height (), 0, 0);
 
         m_blockUpdates = true;
-        win->setGeometry (r);
+        set_geometry (r);
         bar->setVisible (visible);
         m_blockUpdates = false;
 
@@ -977,7 +855,7 @@ namespace QtHandles
   void
   Figure::toggleAxes (void)
   {
-    Canvas* canvas = m_container->canvas (m_handle);
+    Canvas *canvas = m_container->canvas (m_handle);
 
     if (canvas)
       canvas->toggleAxes (m_handle);
@@ -986,7 +864,7 @@ namespace QtHandles
   void
   Figure::toggleGrid (void)
   {
-    Canvas* canvas = m_container->canvas (m_handle);
+    Canvas *canvas = m_container->canvas (m_handle);
 
     if (canvas)
       canvas->toggleGrid (m_handle);
@@ -995,21 +873,58 @@ namespace QtHandles
   void
   Figure::autoAxes (void)
   {
-    Canvas* canvas = m_container->canvas (m_handle);
+    Canvas *canvas = m_container->canvas (m_handle);
 
     if (canvas)
       canvas->autoAxes (m_handle);
   }
 
+  void
+  Figure::figureWindowShown ()
+  {
+#if defined (HAVE_QSCREEN_DEVICEPIXELRATIO)
+    QWindow* window = qWidget<QMainWindow> ()->windowHandle ();
+    QScreen* screen = window->screen ();
+
+    gh_manager::auto_lock lock;
+
+    figure::properties& fp = properties<figure> ();
+    fp.set___device_pixel_ratio__ (screen->devicePixelRatio ());
+
+    connect (window, SIGNAL (screenChanged (QScreen*)),
+             this, SLOT (screenChanged (QScreen*)));
+
+#endif
+  }
+
+  void
+  Figure::screenChanged (QScreen* screen)
+  {
+#if defined (HAVE_QSCREEN_DEVICEPIXELRATIO)
+    gh_manager::auto_lock lock;
+
+    figure::properties& fp = properties<figure> ();
+    double old_dpr = fp.get___device_pixel_ratio__ ();
+    double new_dpr = screen->devicePixelRatio ();
+    if (old_dpr != new_dpr)
+      {
+        fp.set___device_pixel_ratio__ (new_dpr);
+
+        // For some obscure reason, changing the __device_pixel_ratio__ property
+        // from the GUI thread does not necessarily trigger a redraw. Force it.
+        redraw ();
+      }
+#endif
+  }
+
   void
   Figure::enableMouseTracking (void)
   {
     // Enable mouse tracking on every widgets
     m_container->setMouseTracking (true);
     m_container->canvas (m_handle)->qWidget ()->setMouseTracking (true);
-    foreach (QWidget* w, m_container->findChildren<QWidget*> ())
-    { w->setMouseTracking (true); }
+    foreach (QWidget *w, m_container->findChildren<QWidget*> ())
+      w->setMouseTracking (true);
   }
 
 }
-
