@@ -1,22 +1,22 @@
 /*
 
-Copyright (C) 2016-2017 Carnë Draug
+Copyright (C) 2016-2019 Carnë Draug
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
@@ -73,7 +73,7 @@ can have any value real or complex value.  However, for polygamma functions
     print_usage ();
 
   const octave_value oct_z = (nargin == 1) ? args(0) : args(1);
-  const octave_idx_type k = (nargin == 1) ? 0 : args(0).idx_type_value ("psi: K must be an integer");
+  const octave_idx_type k = (nargin == 1) ? 0 : args(0).xidx_type_value ("psi: K must be an integer");
   if (k < 0)
     error ("psi: K must be non-negative");
 
@@ -87,8 +87,8 @@ can have any value real or complex value.  However, for polygamma functions
           const A ## NDArray z = oct_z.M ## array_value ();     \
           A ## NDArray psi_z (z.dims ());                       \
                                                                 \
-          const E* zv = z.data ();                              \
-          E* psi_zv = psi_z.fortran_vec ();                     \
+          const E *zv = z.data ();                              \
+          E *psi_zv = psi_z.fortran_vec ();                     \
           const octave_idx_type n = z.numel ();                 \
           for (octave_idx_type i = 0; i < n; i++)               \
             *psi_zv++ = octave::math::psi (*zv++);              \
@@ -96,7 +96,7 @@ can have any value real or complex value.  However, for polygamma functions
           retval = psi_z;                                       \
         }
 
-      if (oct_z.is_complex_type ())
+      if (oct_z.iscomplex ())
         {
           FLOAT_BRANCH(double, Complex, complex_, Complex)
           else FLOAT_BRANCH(single, FloatComplex, float_complex_, FloatComplex)
@@ -115,7 +115,7 @@ can have any value real or complex value.  However, for polygamma functions
     }
   else
     {
-      if (! oct_z.is_real_type ())
+      if (! oct_z.isreal ())
         error ("psi: Z must be real value for polygamma (K > 0)");
 
 #define FLOAT_BRANCH(T, A, M, E)                                        \
@@ -124,8 +124,8 @@ can have any value real or complex value.  However, for polygamma functions
           const A ## NDArray z = oct_z.M ## array_value ();             \
           A ## NDArray psi_z (z.dims ());                               \
                                                                         \
-          const E* zv = z.data ();                                      \
-          E* psi_zv = psi_z.fortran_vec ();                             \
+          const E *zv = z.data ();                                      \
+          E *psi_zv = psi_z.fortran_vec ();                             \
           const octave_idx_type n = z.numel ();                         \
           for (octave_idx_type i = 0; i < n; i++)                       \
             {                                                           \
@@ -222,7 +222,7 @@ can have any value real or complex value.  However, for polygamma functions
 %!error psi ()
 %!error psi (1, 2, 3)
 %!error <Z must be> psi ("non numeric")
-%!error <conversion of 5.3 to int.* value failed> psi (5.3, 1)
+%!error <K must be an integer> psi ({5.3}, 1)
 %!error <K must be non-negative> psi (-5, 1)
 %!error <Z must be non-negative for polygamma> psi (5, -1)
 %!error <Z must be a floating point> psi (5, uint8 (-1))
